package hdm.mi.sthbackend.serviceTests;

import hdm.mi.sthbackend.exeptions.BracketAlreadyInitializedException;
import hdm.mi.sthbackend.exeptions.InsufficientTeamsException;
import hdm.mi.sthbackend.exeptions.TournamentIdNotFoundException;
import hdm.mi.sthbackend.model.Tournament;
import hdm.mi.sthbackend.repository.ITournamentRepository;
import hdm.mi.sthbackend.service.TournamentService;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.junit.jupiter.MockitoExtension;

import static hdm.mi.sthbackend.dummyObjects.dummyTournaments.*;
import static org.mockito.Mockito.*;

import java.util.Optional;

@ExtendWith(MockitoExtension.class)
public class CreateBracketTest {

    @Mock
    public ITournamentRepository tournamentRepository;
    @InjectMocks
    public TournamentService tournamentService;

    private final Logger log = LogManager.getLogger("CreateBracketTest");

    @Test
    public void create2TeamBracketTest() throws TournamentIdNotFoundException, InsufficientTeamsException, BracketAlreadyInitializedException {
        when(tournamentRepository.findById(dummyTournamentId)).thenReturn(Optional.of(t1));

        Tournament resultTournament = tournamentService.createBracket(dummyTournamentId);
        Assertions.assertEquals(1, resultTournament.getBracket().size());
        Assertions.assertEquals(1, resultTournament.getBracket().get(0).getMatches().size());
    }
    @Test
    public void create4TeamBracketTest() throws TournamentIdNotFoundException, InsufficientTeamsException, BracketAlreadyInitializedException {
        when(tournamentRepository.findById(dummyTournamentId)).thenReturn(Optional.of(t2));

        Tournament resultTournament = tournamentService.createBracket(dummyTournamentId);
        Assertions.assertEquals(2, resultTournament.getBracket().size());
        Assertions.assertEquals(2, resultTournament.getBracket().get(0).getMatches().size());
        Assertions.assertEquals(1, resultTournament.getBracket().get(1).getMatches().size());
    }
    @Test
    public void create8TeamBracketTest() throws TournamentIdNotFoundException, InsufficientTeamsException, BracketAlreadyInitializedException {
        when(tournamentRepository.findById(dummyTournamentId)).thenReturn(Optional.of(t3));

        Tournament resultTournament = tournamentService.createBracket(dummyTournamentId);
        Assertions.assertEquals(3, resultTournament.getBracket().size());
        Assertions.assertEquals(4, resultTournament.getBracket().get(0).getMatches().size());
        Assertions.assertEquals(2, resultTournament.getBracket().get(1).getMatches().size());
        Assertions.assertEquals(1, resultTournament.getBracket().get(2).getMatches().size());
    }
    @Test
    public void create16TeamBracketTest() throws TournamentIdNotFoundException, InsufficientTeamsException, BracketAlreadyInitializedException {
        when(tournamentRepository.findById(dummyTournamentId)).thenReturn(Optional.of(t4));

        Tournament resultTournament = tournamentService.createBracket(dummyTournamentId);
        Assertions.assertEquals(4, resultTournament.getBracket().size());
        Assertions.assertEquals(8, resultTournament.getBracket().get(0).getMatches().size());
        Assertions.assertEquals(4, resultTournament.getBracket().get(1).getMatches().size());
        Assertions.assertEquals(2, resultTournament.getBracket().get(2).getMatches().size());
        Assertions.assertEquals(1, resultTournament.getBracket().get(3).getMatches().size());
    }
    @Test
    public void create1TeamBracketTest(){
        when(tournamentRepository.findById(dummyTournamentId)).thenReturn(Optional.of(t11));

        Assertions.assertThrows(InsufficientTeamsException.class, () -> tournamentService.createBracket(dummyTournamentId));
    }
    @Test
    public void create3TeamBracketTest() throws TournamentIdNotFoundException, InsufficientTeamsException, BracketAlreadyInitializedException {
        when(tournamentRepository.findById(dummyTournamentId)).thenReturn(Optional.of(t12));

        Tournament resultTournament = tournamentService.createBracket(dummyTournamentId);
        Assertions.assertEquals(2, resultTournament.getBracket().size());
        Assertions.assertEquals(2, resultTournament.getBracket().get(0).getMatches().size());
        Assertions.assertEquals(1, resultTournament.getBracket().get(1).getMatches().size());
    }
    @Test
    public void create5TeamBracketTest() throws TournamentIdNotFoundException, InsufficientTeamsException, BracketAlreadyInitializedException {
        when(tournamentRepository.findById(dummyTournamentId)).thenReturn(Optional.of(t13));

        Tournament resultTournament = tournamentService.createBracket(dummyTournamentId);
        Assertions.assertEquals(3, resultTournament.getBracket().size());
        Assertions.assertEquals(4, resultTournament.getBracket().get(0).getMatches().size());
        Assertions.assertEquals(2, resultTournament.getBracket().get(1).getMatches().size());
        Assertions.assertEquals(1, resultTournament.getBracket().get(2).getMatches().size());
    }
    @Test
    public void create15TeamBracketTest() throws TournamentIdNotFoundException, InsufficientTeamsException, BracketAlreadyInitializedException {
        when(tournamentRepository.findById(dummyTournamentId)).thenReturn(Optional.of(t14));

        Tournament resultTournament = tournamentService.createBracket(dummyTournamentId);
        Assertions.assertEquals(4, resultTournament.getBracket().size(), 4);
        Assertions.assertEquals(8, resultTournament.getBracket().get(0).getMatches().size());
        Assertions.assertEquals(4, resultTournament.getBracket().get(1).getMatches().size());
        Assertions.assertEquals(2, resultTournament.getBracket().get(2).getMatches().size());
        Assertions.assertEquals(1, resultTournament.getBracket().get(3).getMatches().size());
    }

}
